@@ -150,11 +150,11 @@ public class QtInputDelegate implements QtInputConnection.QtInputConnectionListe
             return;
         m_keyboardIsVisible = visibility;
         keyboardVisibilityUpdated(m_keyboardIsVisible);
-
-        // Hiding the keyboard clears the immersive mode, so we need to set it again.
-        if (!visibility)
+        if (!visibility) {
+            // Hiding the keyboard clears the immersive mode, so we need to set it again.
             m_keyboardVisibilityListener.onKeyboardVisibilityChange();
-
+            m_currentEditText.clearFocus();
+        }
     }
 
     @UsedFromNativeCode
@@ -185,6 +185,7 @@ public class QtInputDelegate implements QtInputConnection.QtInputConnectionListe
                 return;
 
             m_currentEditText.setEditTextOptions(enterKeyType, inputHints);
+            m_currentEditText.requestFocus();
 
             m_currentEditText.postDelayed(() -> {
                 m_imm.showSoftInput(m_currentEditText, 0, new ResultReceiver(new Handler()) {
@@ -405,12 +406,13 @@ public class QtInputDelegate implements QtInputConnection.QtInputConnectionListe
         if (!QtClipboardManager.hasClipboardText(activity))
             editButtons &= ~EditContextView.PASTE_BUTTON;
 
-        if ((mode & CursorHandleShowEdit) == CursorHandleShowEdit && editButtons != 0) {
-            m_editPopupMenu.setPosition(editX, editY, editButtons,
-                    m_cursorHandle, m_leftSelectionHandle, m_rightSelectionHandle);
-        } else {
-            if (m_editPopupMenu != null)
+        if (m_editPopupMenu != null) {
+            if ((mode & CursorHandleShowEdit) == CursorHandleShowEdit && editButtons != 0) {
+                m_editPopupMenu.setPosition(editX, editY, editButtons,
+                        m_cursorHandle, m_leftSelectionHandle, m_rightSelectionHandle);
+            } else {
                 m_editPopupMenu.hide();
+            }
         }
     }
 
@@ -500,8 +502,8 @@ public class QtInputDelegate implements QtInputConnection.QtInputConnectionListe
     // tablet methods
 
     // pointer methods
-    public static native void mouseDown(int winId, int x, int y);
-    public static native void mouseUp(int winId, int x, int y);
+    public static native void mouseDown(int winId, int x, int y, int mouseButtonState);
+    public static native void mouseUp(int winId, int x, int y, int mouseButtonState);
     public static native void mouseMove(int winId, int x, int y);
     public static native void mouseWheel(int winId, int x, int y, float hDelta, float vDelta);
     public static native void touchBegin(int winId);
@@ -617,11 +619,11 @@ public class QtInputDelegate implements QtInputConnection.QtInputConnectionListe
     {
         switch (event.getActionMasked()) {
             case MotionEvent.ACTION_UP:
-                mouseUp(id, (int) event.getX(), (int) event.getY());
+                mouseUp(id, (int) event.getX(), (int) event.getY(), event.getButtonState());
                 break;
 
             case MotionEvent.ACTION_DOWN:
-                mouseDown(id, (int) event.getX(), (int) event.getY());
+                mouseDown(id, (int) event.getX(), (int) event.getY(), event.getButtonState());
                 m_oldX = (int) event.getX();
                 m_oldY = (int) event.getY();
                 break;
