@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtNetwork module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qbytearray.h"
 #include "qset.h"
@@ -53,11 +17,7 @@
 #  include "qdatetime.h"
 #endif
 
-#if defined(QT_LINUXBASE)
-#  define QT_NO_GETIFADDRS
-#endif
-
-#ifndef QT_NO_GETIFADDRS
+#if QT_CONFIG(getifaddrs)
 # include <ifaddrs.h>
 #endif
 
@@ -92,12 +52,23 @@ static QHostAddress addressFromSockaddr(sockaddr *sa, int ifindex = 0, const QSt
         }
     }
     return address;
+}
 
+template <typename Req> [[maybe_unused]]
+static auto &ifreq_index(Req &req, std::enable_if_t<sizeof(std::declval<Req>().ifr_index) != 0, int> = 0)
+{
+    return req.ifr_index;
+}
+
+template <typename Req> [[maybe_unused]]
+static auto &ifreq_index(Req &req, std::enable_if_t<sizeof(std::declval<Req>().ifr_ifindex) != 0, int> = 0)
+{
+    return req.ifr_ifindex;
 }
 
 uint QNetworkInterfaceManager::interfaceIndexFromName(const QString &name)
 {
-#ifndef QT_NO_IPV6IFNAME
+#if QT_CONFIG(ipv6ifname)
     return ::if_nametoindex(name.toLatin1());
 #elif defined(SIOCGIFINDEX)
     struct ifreq req;
@@ -111,11 +82,7 @@ uint QNetworkInterfaceManager::interfaceIndexFromName(const QString &name)
 
     uint id = 0;
     if (qt_safe_ioctl(socket, SIOCGIFINDEX, &req) >= 0)
-#  if QT_CONFIG(ifr_index)
-        id = req.ifr_index;
-#  else
-        id = req.ifr_ifindex;
-#  endif
+        id = ifreq_index(req);
     qt_safe_close(socket);
     return id;
 #else
@@ -126,7 +93,7 @@ uint QNetworkInterfaceManager::interfaceIndexFromName(const QString &name)
 
 QString QNetworkInterfaceManager::interfaceNameFromIndex(uint index)
 {
-#ifndef QT_NO_IPV6IFNAME
+#if QT_CONFIG(ipv6ifname)
     char buf[IF_NAMESIZE];
     if (::if_indextoname(index, buf))
         return QString::fromLatin1(buf);
@@ -135,12 +102,7 @@ QString QNetworkInterfaceManager::interfaceNameFromIndex(uint index)
     int socket = qt_safe_socket(AF_INET, SOCK_STREAM, 0);
     if (socket >= 0) {
         memset(&req, 0, sizeof(ifreq));
-#  if QT_CONFIG(ifr_index)
-        req.ifr_index = index;
-#  else
-        req.ifr_ifindex = index;
-#  endif
-
+        ifreq_index(req) = index;
         if (qt_safe_ioctl(socket, SIOCGIFNAME, &req) >= 0) {
             qt_safe_close(socket);
             return QString::fromLatin1(req.ifr_name);
@@ -160,13 +122,13 @@ static int getMtu(int socket, struct ifreq *req)
     return 0;
 }
 
-#ifdef QT_NO_GETIFADDRS
+#if !QT_CONFIG(getifaddrs)
 // getifaddrs not available
 
 static QSet<QByteArray> interfaceNames(int socket)
 {
     QSet<QByteArray> result;
-#ifdef QT_NO_IPV6IFNAME
+#if !QT_CONFIG(ipv6ifname)
     QByteArray storageBuffer;
     struct ifconf interfaceList;
     static const int STORAGEBUFFER_GROWTH = 256;
@@ -221,15 +183,11 @@ static QNetworkInterfacePrivate *findInterface(int socket, QList<QNetworkInterfa
     QNetworkInterfacePrivate *iface = nullptr;
     int ifindex = 0;
 
-#if !defined(QT_NO_IPV6IFNAME) || defined(SIOCGIFINDEX)
+#if QT_CONFIG(ipv6ifname) || defined(SIOCGIFINDEX)
     // Get the interface index
 #  ifdef SIOCGIFINDEX
     if (qt_safe_ioctl(socket, SIOCGIFINDEX, &req) >= 0)
-#    if QT_CONFIG(ifr_index)
-        ifindex = req.ifr_index;
-#    else
-        ifindex = req.ifr_ifindex;
-#    endif
+        ifindex = ifreq_index(req);
 #  else
     ifindex = if_nametoindex(req.ifr_name);
 #  endif
@@ -247,7 +205,7 @@ static QNetworkInterfacePrivate *findInterface(int socket, QList<QNetworkInterfa
     // Search by name
     QList<QNetworkInterfacePrivate *>::Iterator if_it = interfaces.begin();
     for ( ; if_it != interfaces.end(); ++if_it)
-        if ((*if_it)->name == QLatin1String(req.ifr_name)) {
+        if ((*if_it)->name == QLatin1StringView(req.ifr_name)) {
             // existing interface
             iface = *if_it;
             break;
@@ -418,10 +376,7 @@ QT_BEGIN_INCLUDE_NAMESPACE
 #  include <net/if_dl.h>
 #if defined(QT_PLATFORM_UIKIT)
 #  include "qnetworkinterface_uikit_p.h"
-#if !defined(QT_WATCHOS_OUTDATED_SDK_WORKAROUND)
-// TODO: remove it as soon as SDK is updated on CI!!!
 #  include <net/if_types.h>
-#endif
 #else
 #  include <net/if_media.h>
 #  include <net/if_types.h>
@@ -498,7 +453,8 @@ static QList<QNetworkInterfacePrivate *> createInterfaces(ifaddrs *rawList)
 
     // ensure both structs start with the name field, of size IFNAMESIZ
     static_assert(sizeof(mediareq.ifm_name) == sizeof(req.ifr_name));
-    Q_ASSERT(&mediareq.ifm_name == &req.ifr_name);
+    static_assert(offsetof(struct ifmediareq, ifm_name) == 0);
+    static_assert(offsetof(struct ifreq, ifr_name) == 0);
 
     // on NetBSD we use AF_LINK and sockaddr_dl
     // scan the list for that family
@@ -627,7 +583,7 @@ static QList<QNetworkInterfacePrivate *> interfaceListing()
     interfaces = createInterfaces(interfaceListing);
     for (ifaddrs *ptr = interfaceListing; ptr; ptr = ptr->ifa_next) {
         // Find the interface
-        QLatin1String name(ptr->ifa_name);
+        QLatin1StringView name(ptr->ifa_name);
         QNetworkInterfacePrivate *iface = nullptr;
         QList<QNetworkInterfacePrivate *>::Iterator if_it = interfaces.begin();
         for ( ; if_it != interfaces.end(); ++if_it)
