@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 /****************************************************************************
 **
@@ -71,25 +35,30 @@
 **
 ****************************************************************************/
 
+#include <AppKit/AppKit.h>
+
 #include "qcocoaeventdispatcher.h"
 #include "qcocoawindow.h"
-
 #include "qcocoahelpers.h"
-#include "qguiapplication.h"
-#include "qevent.h"
-#include "qmutex.h"
-#include "qsocketnotifier.h"
+
+#include <QtGui/qevent.h>
+#include <QtGui/qguiapplication.h>
+#include <QtGui/private/qguiapplication_p.h>
+
+#include <QtCore/qmutex.h>
+#include <QtCore/qscopeguard.h>
+#include <QtCore/qsocketnotifier.h>
+#include <QtCore/private/qthread_p.h>
+#include <QtCore/private/qcore_mac_p.h>
+
 #include <qpa/qplatformwindow.h>
 #include <qpa/qplatformnativeinterface.h>
-#include "private/qthread_p.h"
-#include "private/qguiapplication_p.h"
-#include <qdebug.h>
 
-#include <AppKit/AppKit.h>
+#include <QtCore/qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
-QT_USE_NAMESPACE
+Q_LOGGING_CATEGORY(lcEventDispatcher, "qt.eventdispatcher");
 
 static inline CFRunLoopRef mainRunLoop()
 {
@@ -120,6 +89,13 @@ void QCocoaEventDispatcherPrivate::runLoopTimerCallback(CFRunLoopTimerRef, void
 void QCocoaEventDispatcherPrivate::activateTimersSourceCallback(void *info)
 {
     QCocoaEventDispatcherPrivate *d = static_cast<QCocoaEventDispatcherPrivate *>(info);
+    if (d->initializingNSApplication) {
+        qCDebug(lcEventDispatcher) << "Deferring" << __FUNCTION__ << "due to NSApp initialization";
+        // We don't want to process any sources during explicit NSApplication
+        // initialization, so defer the source until the actual event processing.
+        CFRunLoopSourceSignal(d->activateTimersSourceRef);
+        return;
+    }
     d->processTimers();
     d->maybeCancelWaitForMoreEvents();
 }
@@ -139,6 +115,7 @@ void QCocoaEventDispatcherPrivate::maybeStartCFRunLoopTimer()
         return;
     }
 
+    using DoubleSeconds = std::chrono::duration<double, std::ratio<1>>;
     if (!runLoopTimerRef) {
         // start the CFRunLoopTimer
         CFAbsoluteTime ttf = CFAbsoluteTimeGetCurrent();
@@ -146,10 +123,10 @@ void QCocoaEventDispatcherPrivate::maybeStartCFRunLoopTimer()
         CFTimeInterval oneyear = CFTimeInterval(3600. * 24. * 365.);
 
         // Q: when should the CFRunLoopTimer fire for the first time?
-        struct timespec tv;
-        if (timerInfoList.timerWait(tv)) {
+        if (auto opt = timerInfoList.timerWait()) {
             // A: when we have timers to fire, of course
-            interval = qMax(tv.tv_sec + tv.tv_nsec / 1000000000., 0.0000001);
+            DoubleSeconds secs{*opt};
+            interval = qMax(secs.count(), 0.0000001);
         } else {
             // this shouldn't really happen, but in case it does, set the timer to fire a some point in the distant future
             interval = oneyear;
@@ -169,10 +146,10 @@ void QCocoaEventDispatcherPrivate::maybeStartCFRunLoopTimer()
         CFTimeInterval interval;
 
         // Q: when should the timer first next?
-        struct timespec tv;
-        if (timerInfoList.timerWait(tv)) {
+        if (auto opt = timerInfoList.timerWait()) {
             // A: when we have timers to fire, of course
-            interval = qMax(tv.tv_sec + tv.tv_nsec / 1000000000., 0.0000001);
+            DoubleSeconds secs{*opt};
+            interval = qMax(secs.count(), 0.0000001);
         } else {
             // no timers can fire, but we cannot stop the CFRunLoopTimer, set the timer to fire at some
             // point in the distant future (the timer interval is one year)
@@ -194,7 +171,7 @@ void QCocoaEventDispatcherPrivate::maybeStopCFRunLoopTimer()
     runLoopTimerRef = nullptr;
 }
 
-void QCocoaEventDispatcher::registerTimer(int timerId, int interval, Qt::TimerType timerType, QObject *obj)
+void QCocoaEventDispatcher::registerTimer(int timerId, qint64 interval, Qt::TimerType timerType, QObject *obj)
 {
 #ifndef QT_NO_DEBUG
     if (timerId < 1 || interval < 0 || !obj) {
@@ -286,53 +263,42 @@ void QCocoaEventDispatcher::unregisterSocketNotifier(QSocketNotifier *notifier)
     d->cfSocketNotifier.unregisterSocketNotifier(notifier);
 }
 
-bool QCocoaEventDispatcher::hasPendingEvents()
+static bool isUserInputEvent(NSEvent* event)
 {
-    extern uint qGlobalPostedEventsCount();
-    extern bool qt_is_gui_used; //qapplication.cpp
-    return qGlobalPostedEventsCount() || (qt_is_gui_used && !CFRunLoopIsWaiting(CFRunLoopGetMain()));
-}
-
-static bool IsMouseOrKeyEvent( NSEvent* event )
-{
-    bool    result    = false;
-
-    switch( [event type] )
-    {
-        case NSEventTypeLeftMouseDown:
-        case NSEventTypeLeftMouseUp:
-        case NSEventTypeRightMouseDown:
-        case NSEventTypeRightMouseUp:
-        case NSEventTypeMouseMoved:                // ??
-        case NSEventTypeLeftMouseDragged:
-        case NSEventTypeRightMouseDragged:
-        case NSEventTypeMouseEntered:
-        case NSEventTypeMouseExited:
-        case NSEventTypeKeyDown:
-        case NSEventTypeKeyUp:
-        case NSEventTypeFlagsChanged:            // key modifiers changed?
-        case NSEventTypeCursorUpdate:            // ??
-        case NSEventTypeScrollWheel:
-        case NSEventTypeTabletPoint:
-        case NSEventTypeTabletProximity:
-        case NSEventTypeOtherMouseDown:
-        case NSEventTypeOtherMouseUp:
-        case NSEventTypeOtherMouseDragged:
+    switch ([event type]) {
+    case NSEventTypeLeftMouseDown:
+    case NSEventTypeLeftMouseUp:
+    case NSEventTypeRightMouseDown:
+    case NSEventTypeRightMouseUp:
+    case NSEventTypeMouseMoved:                // ??
+    case NSEventTypeLeftMouseDragged:
+    case NSEventTypeRightMouseDragged:
+    case NSEventTypeMouseEntered:
+    case NSEventTypeMouseExited:
+    case NSEventTypeKeyDown:
+    case NSEventTypeKeyUp:
+    case NSEventTypeFlagsChanged:            // key modifiers changed?
+    case NSEventTypeCursorUpdate:            // ??
+    case NSEventTypeScrollWheel:
+    case NSEventTypeTabletPoint:
+    case NSEventTypeTabletProximity:
+    case NSEventTypeOtherMouseDown:
+    case NSEventTypeOtherMouseUp:
+    case NSEventTypeOtherMouseDragged:
 #ifndef QT_NO_GESTURES
-        case NSEventTypeGesture: // touch events
-        case NSEventTypeMagnify:
-        case NSEventTypeSwipe:
-        case NSEventTypeRotate:
-        case NSEventTypeBeginGesture:
-        case NSEventTypeEndGesture:
+    case NSEventTypeGesture: // touch events
+    case NSEventTypeMagnify:
+    case NSEventTypeSwipe:
+    case NSEventTypeRotate:
+    case NSEventTypeBeginGesture:
+    case NSEventTypeEndGesture:
 #endif // QT_NO_GESTURES
-            result    = true;
+        return true;
         break;
-
-        default:
+    default:
         break;
     }
-    return result;
+    return false;
 }
 
 static inline void qt_mac_waitForMoreEvents(NSString *runLoopMode = NSDefaultRunLoopMode)
@@ -352,6 +318,16 @@ static inline void qt_mac_waitForMoreEvents(NSString *runLoopMode = NSDefaultRun
 bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
 {
     Q_D(QCocoaEventDispatcher);
+
+    // In rare rather corner cases a user's application messes with
+    // QEventLoop::exec()/exit() and QCoreApplication::processEvents(),
+    // we have to undo what bool blocker normally does.
+    d->propagateInterrupt = false;
+    const auto boolBlockerUndo = qScopeGuard([d](){
+        if (d->propagateInterrupt)
+            d->interrupt = true;
+        d->propagateInterrupt = false;
+    });
     QBoolBlocker interruptBlocker(d->interrupt, false);
 
     bool interruptLater = false;
@@ -377,16 +353,9 @@ bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
         NSEvent* event = nil;
 
         // First, send all previously excluded input events, if any:
-        if (!excludeUserEvents) {
-            while (!d->queuedUserInputEvents.isEmpty()) {
-                event = static_cast<NSEvent *>(d->queuedUserInputEvents.takeFirst());
-                if (!filterNativeEvent("NSEvent", event, nullptr)) {
-                    [NSApp sendEvent:event];
-                    retVal = true;
-                }
-                [event release];
-            }
-        }
+        if (d->sendQueuedUserInputEvents())
+            retVal = true;
+
 
         // If Qt is used as a plugin, or as an extension in a native cocoa
         // application, we should not run or stop NSApplication; This will be
@@ -406,6 +375,7 @@ bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
             // [NSApp run], which is the normal code path for cocoa applications.
             if (NSModalSession session = d->currentModalSession()) {
                 QBoolBlocker execGuard(d->currentExecIsNSAppRun, false);
+                qCDebug(lcEventDispatcher) << "Running modal session" << session;
                 while ([NSApp runModalSession:session] == NSModalResponseContinue && !d->interrupt) {
                     qt_mac_waitForMoreEvents(NSModalPanelRunLoopMode);
                     if (session != d->currentModalSessionCached) {
@@ -449,6 +419,7 @@ bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
                     // to use cocoa's native way of running modal sessions:
                     if (flags & QEventLoop::WaitForMoreEvents)
                         qt_mac_waitForMoreEvents(NSModalPanelRunLoopMode);
+                    qCDebug(lcEventDispatcher) << "Running modal session" << session;
                     NSInteger status = [NSApp runModalSession:session];
                     if (status != NSModalResponseContinue && session == d->currentModalSessionCached) {
                         // INVARIANT: Someone called [NSApp stopModal:] from outside the event
@@ -472,7 +443,7 @@ bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
                     dequeue: YES];
 
                     if (event) {
-                        if (IsMouseOrKeyEvent(event)) {
+                        if (isUserInputEvent(event)) {
                             [event retain];
                             d->queuedUserInputEvents.append(event);
                             continue;
@@ -492,7 +463,7 @@ bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
 
                 if (event) {
                     if (flags & QEventLoop::ExcludeUserInputEvents) {
-                        if (IsMouseOrKeyEvent(event)) {
+                        if (isUserInputEvent(event)) {
                             [event retain];
                             d->queuedUserInputEvents.append(event);
                             continue;
@@ -503,11 +474,25 @@ bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
                         retVal = true;
                     }
                 }
+
+                // Clean up the modal session list, call endModalSession.
+                if (d->cleanupModalSessionsNeeded)
+                    d->cleanupModalSessions();
+
             } while (!d->interrupt && event);
 
             if ((d->processEventsFlags & QEventLoop::EventLoopExec) == 0) {
                 // When called "manually", always process posted events and timers
+                bool oldInterrupt = d->interrupt;
                 d->processPostedEvents();
+                if (!oldInterrupt && d->interrupt && !d->currentModalSession()) {
+                    // We had direct processEvent call, coming not from QEventLoop::exec().
+                    // One of the posted events triggered an application to interrupt the loop.
+                    // But bool blocker will reset d->interrupt to false, so the real event
+                    // loop will never notice it was interrupted. Now we'll have to fix it by
+                    // enforcing the value of d->interrupt.
+                    d->propagateInterrupt = true;
+                }
                 retVal = d->processTimers() || retVal;
             }
 
@@ -522,7 +507,7 @@ bool QCocoaEventDispatcher::processEvents(QEventLoop::ProcessEventsFlags flags)
             if (hadModalSession && !d->currentModalSessionCached)
                 interruptLater = true;
         }
-        bool canWait = (d->threadData->canWait
+        bool canWait = (d->threadData.loadRelaxed()->canWait
                 && !retVal
                 && !d->interrupt
                 && (d->processEventsFlags & QEventLoop::WaitForMoreEvents));
@@ -582,22 +567,42 @@ void QCocoaEventDispatcher::wakeUp()
 
 void QCocoaEventDispatcherPrivate::ensureNSAppInitialized()
 {
-    // Some elements in Cocoa require NSApplication to be running before
-    // they get fully initialized, in particular the menu bar. This
-    // function is intended for cases where a dialog is told to execute before
-    // QGuiApplication::exec is called, or the application spins the events loop
-    // manually rather than calling QGuiApplication:exec.
-    // The function makes sure that NSApplication starts running, but stops
-    // it again as soon as the send posted events callback is called. That way
-    // we let Cocoa finish the initialization it seems to need. We'll only
-    // apply this trick at most once for any application, and we avoid doing it
-    // for the common case where main just starts QGuiApplication::exec.
+    // Some elements in Cocoa require NSApplication to be initialized before
+    // use, for example the menu bar. Under normal circumstances this happens
+    // as part of [NSApp run], as a result of a call to QGuiApplication:exec(),
+    // but in the cases where a dialog is asked to execute before that happens,
+    // or the application spins the event loop manually via processEvents(),
+    // we need to explicitly ensure NSApplication initialization.
+
+    // We can unfortunately not do this via NSApplicationLoad(), as the function
+    // bails out early if there's already an NSApplication instance, which is
+    // the case if any code has called [NSApplication sharedApplication],
+    // or its short form 'NSApp'.
+
+    // Instead we do an actual [NSApp run], but stop the application as soon
+    // as possible, ensuring that AppKit will do the required initialization,
+    // including calling [NSApplication finishLaunching].
+
+    // We only apply this trick at most once for any application, and we avoid
+    // doing it for the common case where main just starts QGuiApplication::exec.
     if (nsAppRunCalledByQt || [NSApp isRunning])
         return;
+
+    qCDebug(lcEventDispatcher) << "Ensuring NSApplication is initialized";
     nsAppRunCalledByQt = true;
-    QBoolBlocker block1(interrupt, true);
-    QBoolBlocker block2(currentExecIsNSAppRun, true);
+
+    // Stopping the application will still process runloop sources before
+    // actually stopping, so we need to explicitly guard our sources from
+    // doing anything, deferring their actions until later.
+    QBoolBlocker initializationGuard(initializingNSApplication, true);
+
+    CFRunLoopPerformBlock(mainRunLoop(), kCFRunLoopCommonModes, ^{
+        qCDebug(lcEventDispatcher) << "NSApplication has been initialized; Stopping NSApp";
+        [NSApp stop:NSApp];
+        cancelWaitForMoreEvents(); // Post event that wakes up the runloop
+    });
     [NSApp run];
+    qCDebug(lcEventDispatcher) << "Finished ensuring NSApplication is initialized";
 }
 
 void QCocoaEventDispatcherPrivate::temporarilyStopAllModalSessions()
@@ -615,6 +620,8 @@ void QCocoaEventDispatcherPrivate::temporarilyStopAllModalSessions()
     for (int i=0; i<stackSize; ++i) {
         QCocoaModalSessionInfo &info = cocoaModalSessionStack[i];
         if (info.session) {
+            qCDebug(lcEventDispatcher) << "Temporarily ending modal session" << info.session
+                                       << "for" << info.nswindow;
             [NSApp endModalSession:info.session];
             info.session = nullptr;
             [(NSWindow*) info.nswindow release];
@@ -654,6 +661,8 @@ NSModalSession QCocoaEventDispatcherPrivate::currentModalSession()
             [(NSWindow*) info.nswindow retain];
             QRect rect = cocoaWindow->geometry();
             info.session = [NSApp beginModalSessionForWindow:nswindow];
+            qCDebug(lcEventDispatcher) << "Begun modal session" << info.session
+                                       << "for" << nswindow;
 
             // The call to beginModalSessionForWindow above processes events and may
             // have deleted or destroyed the window. Check if it's still valid.
@@ -672,49 +681,15 @@ NSModalSession QCocoaEventDispatcherPrivate::currentModalSession()
     return currentModalSessionCached;
 }
 
-static void setChildrenWorksWhenModal(QWindow *window, bool worksWhenModal)
+bool QCocoaEventDispatcherPrivate::hasModalSession() const
 {
-    Q_UNUSED(window)
-    Q_UNUSED(worksWhenModal)
-
-    // For NSPanels (but not NSWindows, sadly), we can set the flag
-    // worksWhenModal, so that they are active even when they are not modal.
-/*
-    ### not ported
-    QList<QDialog *> dialogs = window->findChildren<QDialog *>();
-    for (int i=0; i<dialogs.size(); ++i){
-        NSWindow *window = qt_mac_window_for(dialogs[i]);
-        if (window && [window isKindOfClass:[NSPanel class]]) {
-            [static_cast<NSPanel *>(window) setWorksWhenModal:worksWhenModal];
-            if (worksWhenModal && [window isVisible]){
-                [window orderFront:window];
-            }
-        }
-    }
-*/
-}
-
-void QCocoaEventDispatcherPrivate::updateChildrenWorksWhenModal()
-{
-    // Make the dialog children of the window
-    // active. And make the dialog children of
-    // the previous modal dialog unactive again:
-    QMacAutoReleasePool pool;
-    int size = cocoaModalSessionStack.size();
-    if (size > 0){
-        if (QWindow *prevModal = cocoaModalSessionStack[size-1].window)
-            setChildrenWorksWhenModal(prevModal, true);
-        if (size > 1){
-            if (QWindow *prevModal = cocoaModalSessionStack[size-2].window)
-                setChildrenWorksWhenModal(prevModal, false);
-        }
-    }
+    return !cocoaModalSessionStack.isEmpty();
 }
 
 void QCocoaEventDispatcherPrivate::cleanupModalSessions()
 {
     // Go through the list of modal sessions, and end those
-    // that no longer has a window assosiated; no window means
+    // that no longer has a window associated; no window means
     // the session has logically ended. The reason we wait like
     // this to actually end the sessions for real (rather than at the
     // point they were marked as stopped), is that ending a session
@@ -736,6 +711,8 @@ void QCocoaEventDispatcherPrivate::cleanupModalSessions()
         currentModalSessionCached = nullptr;
         if (info.session) {
             Q_ASSERT(info.nswindow);
+            qCDebug(lcEventDispatcher) << "Ending modal session" << info.session
+                                       << "for" << info.nswindow;
             [NSApp endModalSession:info.session];
             [(NSWindow *)info.nswindow release];
         }
@@ -743,12 +720,19 @@ void QCocoaEventDispatcherPrivate::cleanupModalSessions()
         cocoaModalSessionStack.remove(i);
     }
 
-    updateChildrenWorksWhenModal();
     cleanupModalSessionsNeeded = false;
 }
 
 void QCocoaEventDispatcherPrivate::beginModalSession(QWindow *window)
 {
+    qCDebug(lcEventDispatcher) << "Adding modal session for" << window;
+
+    if (std::any_of(cocoaModalSessionStack.constBegin(), cocoaModalSessionStack.constEnd(),
+        [&](const auto &sessionInfo) { return sessionInfo.window == window; })) {
+        qCWarning(lcEventDispatcher) << "Modal session for" << window << "already exists!";
+        return;
+    }
+
     // We need to start spinning the modal session. Usually this is done with
     // QDialog::exec() for Qt Widgets based applications, but for others that
     // just call show(), we need to interrupt().
@@ -764,12 +748,13 @@ void QCocoaEventDispatcherPrivate::beginModalSession(QWindow *window)
     // stopped in cleanupModalSessions()).
     QCocoaModalSessionInfo info = {window, nullptr, nullptr};
     cocoaModalSessionStack.push(info);
-    updateChildrenWorksWhenModal();
     currentModalSessionCached = nullptr;
 }
 
 void QCocoaEventDispatcherPrivate::endModalSession(QWindow *window)
 {
+    qCDebug(lcEventDispatcher) << "Removing modal session for" << window;
+
     Q_Q(QCocoaEventDispatcher);
 
     // Mark all sessions attached to window as pending to be stopped. We do this
@@ -853,21 +838,6 @@ QCocoaEventDispatcher::QCocoaEventDispatcher(QObject *parent)
                                                  QCocoaEventDispatcherPrivate::waitingObserverCallback,
                                                  &observerContext);
     CFRunLoopAddObserver(mainRunLoop(), d->waitingObserver, kCFRunLoopCommonModes);
-
-    /* The first cycle in the loop adds the source and the events of the source
-       are not processed.
-       We use an observer to process the posted events for the first
-       execution of the loop. */
-    CFRunLoopObserverContext firstTimeObserverContext;
-    bzero(&firstTimeObserverContext, sizeof(CFRunLoopObserverContext));
-    firstTimeObserverContext.info = d;
-    d->firstTimeObserver = CFRunLoopObserverCreate(kCFAllocatorDefault,
-                                                   kCFRunLoopEntry,
-                                                   /* repeats = */ false,
-                                                   0,
-                                                   QCocoaEventDispatcherPrivate::firstLoopEntry,
-                                                   &firstTimeObserverContext);
-    CFRunLoopAddObserver(mainRunLoop(), d->firstTimeObserver, kCFRunLoopCommonModes);
 }
 
 void QCocoaEventDispatcherPrivate::waitingObserverCallback(CFRunLoopObserverRef,
@@ -879,6 +849,23 @@ void QCocoaEventDispatcherPrivate::waitingObserverCallback(CFRunLoopObserverRef,
         emit static_cast<QCocoaEventDispatcher*>(info)->awake();
 }
 
+bool QCocoaEventDispatcherPrivate::sendQueuedUserInputEvents()
+{
+    Q_Q(QCocoaEventDispatcher);
+    if (processEventsFlags & QEventLoop::ExcludeUserInputEvents)
+        return false;
+    bool didSendEvent = false;
+    while (!queuedUserInputEvents.isEmpty()) {
+        NSEvent *event = static_cast<NSEvent *>(queuedUserInputEvents.takeFirst());
+        if (!q->filterNativeEvent("NSEvent", event, nullptr)) {
+            [NSApp sendEvent:event];
+            didSendEvent = true;
+        }
+        [event release];
+    }
+    return didSendEvent;
+}
+
 void QCocoaEventDispatcherPrivate::processPostedEvents()
 {
     if (blockSendPostedEvents) {
@@ -907,31 +894,31 @@ void QCocoaEventDispatcherPrivate::processPostedEvents()
         return;
     }
 
-    int serial = serialNumber.load();
-    if (!threadData->canWait || (serial != lastSerial)) {
+    int serial = serialNumber.loadRelaxed();
+    if (!threadData.loadRelaxed()->canWait || (serial != lastSerial)) {
         lastSerial = serial;
         QCoreApplication::sendPostedEvents();
         QWindowSystemInterface::sendWindowSystemEvents(QEventLoop::AllEvents);
     }
 }
 
-void QCocoaEventDispatcherPrivate::firstLoopEntry(CFRunLoopObserverRef ref,
-                                                CFRunLoopActivity activity,
-                                                void *info)
-{
-    Q_UNUSED(ref);
-    Q_UNUSED(activity);
-    static_cast<QCocoaEventDispatcherPrivate *>(info)->processPostedEvents();
-}
-
 void QCocoaEventDispatcherPrivate::postedEventsSourceCallback(void *info)
 {
     QCocoaEventDispatcherPrivate *d = static_cast<QCocoaEventDispatcherPrivate *>(info);
+    if (d->initializingNSApplication) {
+        qCDebug(lcEventDispatcher) << "Deferring" << __FUNCTION__ << "due to NSApp initialization";
+        // We don't want to process any sources during explicit NSApplication
+        // initialization, so defer the source until the actual event processing.
+        CFRunLoopSourceSignal(d->postedEventsSource);
+        return;
+    }
+
     if (d->processEventsCalled && (d->processEventsFlags & QEventLoop::EventLoopExec) == 0) {
         // processEvents() was called "manually," ignore this source for now
         d->maybeCancelWaitForMoreEvents();
         return;
     }
+    d->sendQueuedUserInputEvents();
     d->processPostedEvents();
     d->maybeCancelWaitForMoreEvents();
 }
@@ -972,9 +959,6 @@ void QCocoaEventDispatcher::interrupt()
     d->cancelWaitForMoreEvents();
 }
 
-void QCocoaEventDispatcher::flush()
-{ }
-
 // QTBUG-56746: The behavior of processEvents() has been changed to not clear
 // the interrupt flag. Use this function to clear it.
  void QCocoaEventDispatcher::clearCurrentThreadCocoaEventDispatcherInterruptFlag()
@@ -992,7 +976,7 @@ QCocoaEventDispatcher::~QCocoaEventDispatcher()
 {
     Q_D(QCocoaEventDispatcher);
 
-    qDeleteAll(d->timerInfoList);
+    d->timerInfoList.clearTimers();
     d->maybeStopCFRunLoopTimer();
     CFRunLoopRemoveSource(mainRunLoop(), d->activateTimersSourceRef, kCFRunLoopCommonModes);
     CFRelease(d->activateTimersSourceRef);
@@ -1001,6 +985,8 @@ QCocoaEventDispatcher::~QCocoaEventDispatcher()
     for (int i = 0; i < d->cocoaModalSessionStack.count(); ++i) {
         QCocoaModalSessionInfo &info = d->cocoaModalSessionStack[i];
         if (info.session) {
+            qCDebug(lcEventDispatcher) << "Ending modal session" << info.session
+                                       << "for" << info.nswindow << "during shutdown";
             [NSApp endModalSession:info.session];
             [(NSWindow *)info.nswindow release];
         }
@@ -1019,9 +1005,6 @@ QCocoaEventDispatcher::~QCocoaEventDispatcher()
 
     CFRunLoopObserverInvalidate(d->waitingObserver);
     CFRelease(d->waitingObserver);
-
-    CFRunLoopObserverInvalidate(d->firstTimeObserver);
-    CFRelease(d->firstTimeObserver);
 }
 
 QtCocoaInterruptDispatcher* QtCocoaInterruptDispatcher::instance = nullptr;
