@@ -30,14 +30,7 @@
 #define _int64 __int64
 #endif
 
-
 #include <oci.h>
-#ifdef max
-#undef max
-#endif
-#ifdef min
-#undef min
-#endif
 
 #include <stdlib.h>
 
@@ -141,7 +134,7 @@ QOCIDateTime::QOCIDateTime(OCIEnv *env, OCIError *err, const QDateTime &dt)
         const QDate date = dt.date();
         const QTime time = dt.time();
         // Zone in +hh:mm format
-        const QString timeZone = dt.toString("ttt");
+        const QString timeZone = dt.toString("ttt"_L1);
         const OraText *tz = reinterpret_cast<const OraText *>(timeZone.utf16());
         OCIDateTimeConstruct(env, err, dateTime, date.year(), date.month(), date.day(), time.hour(),
                              time.minute(), time.second(), time.msec() * 1000000,
@@ -170,7 +163,7 @@ QDateTime QOCIDateTime::fromOCIDateTime(OCIEnv *env, OCIError *err, OCIDateTime
         secondsOffset = -secondsOffset;
     // OCIDateTimeGetTime gives "fractions of second" as nanoseconds
     return QDateTime(QDate(year, month, day), QTime(hour, minute, second, nsec / 1000000),
-                     Qt::OffsetFromUTC, secondsOffset);
+                     QTimeZone::fromSecondsAheadOfUtc(secondsOffset));
 }
 
 struct TempStorage {
@@ -196,6 +189,7 @@ public:
     OCISession *authp = nullptr;
     OCITrans *trans = nullptr;
     OCIError *err = nullptr;
+    ub4 authMode = OCI_DEFAULT;
     bool transaction = false;
     int serverVersion = -1;
     int prefetchRows = -1;
@@ -335,13 +329,13 @@ int QOCIResultPrivate::bindValue(OCIStmt *sql, OCIBind **hbnd, OCIError *err, in
 
     switch (val.typeId()) {
     case QMetaType::QByteArray:
-        r = OCIBindByPos(sql, hbnd, err,
-                         pos + 1,
-                         isOutValue(pos)
+        r = OCIBindByPos2(sql, hbnd, err,
+                          pos + 1,
+                          isOutValue(pos)
                             ?  const_cast<char *>(reinterpret_cast<QByteArray *>(data)->constData())
                             : reinterpret_cast<QByteArray *>(data)->data(),
-                         reinterpret_cast<QByteArray *>(data)->size(),
-                         SQLT_BIN, indPtr, 0, 0, 0, 0, OCI_DEFAULT);
+                          reinterpret_cast<QByteArray *>(data)->size(),
+                          SQLT_BIN, indPtr, 0, 0, 0, 0, OCI_DEFAULT);
         break;
     case QMetaType::QTime:
     case QMetaType::QDate:
@@ -407,20 +401,20 @@ int QOCIResultPrivate::bindValue(OCIStmt *sql, OCIBind **hbnd, OCIError *err, in
     case QMetaType::QString: {
         const QString s = val.toString();
         if (isBinaryValue(pos)) {
-            r = OCIBindByPos(sql, hbnd, err,
-                             pos + 1,
-                             const_cast<ushort *>(s.utf16()),
-                             s.length() * sizeof(QChar),
-                             SQLT_LNG, indPtr, 0, 0, 0, 0, OCI_DEFAULT);
+            r = OCIBindByPos2(sql, hbnd, err,
+                              pos + 1,
+                              const_cast<ushort *>(s.utf16()),
+                              s.length() * sizeof(QChar),
+                              SQLT_LNG, indPtr, 0, 0, 0, 0, OCI_DEFAULT);
             break;
         } else if (!isOutValue(pos)) {
             // don't detach the string
-            r = OCIBindByPos(sql, hbnd, err,
-                             pos + 1,
-                             // safe since oracle doesn't touch OUT values
-                             const_cast<ushort *>(s.utf16()),
-                             (s.length() + 1) * sizeof(QChar),
-                             SQLT_STR, indPtr, 0, 0, 0, 0, OCI_DEFAULT);
+            r = OCIBindByPos2(sql, hbnd, err,
+                              pos + 1,
+                              // safe since oracle doesn't touch OUT values
+                              const_cast<ushort *>(s.utf16()),
+                              (s.length() + 1) * sizeof(QChar),
+                              SQLT_STR, indPtr, 0, 0, 0, 0, OCI_DEFAULT);
             if (r == OCI_SUCCESS)
                 setCharset(*hbnd, OCI_HTYPE_BIND);
             break;
@@ -1435,7 +1429,7 @@ bool QOCICols::execBatch(QOCIResultPrivate *d, QVariantList &boundValues, bool a
 
         // we may now populate column with data
         for (uint row = 0; row < col.recordCount; ++row) {
-            const QVariant &val = boundValues.at(i).toList().at(row);
+            const QVariant val = boundValues.at(i).toList().at(row);
 
             if (QSqlResultPrivate::isVariantNull(val) && !d->isOutValue(i)) {
                 columns[i].indicators[row] = -1;
@@ -1486,7 +1480,7 @@ bool QOCICols::execBatch(QOCIResultPrivate *d, QVariantList &boundValues, bool a
 
                     case QMetaType::QString: {
                         const QString s = val.toString();
-                        columns[i].lengths[row] = (s.length() + 1) * sizeof(QChar);
+                        columns[i].lengths[row] = ub2((s.length() + 1) * sizeof(QChar));
                         memcpy(dataPtr, s.utf16(), columns[i].lengths[row]);
                         break;
                     }
@@ -1963,7 +1957,7 @@ bool QOCIResult::prepare(const QString& query)
     }
     d->setStatementAttributes();
     const OraText *txt = reinterpret_cast<const OraText *>(query.utf16());
-    const int len = query.length() * sizeof(QChar);
+    const auto len = ub4(query.length() * sizeof(QChar));
     r = OCIStmtPrepare(d->sql,
                        d->err,
                        txt,
@@ -2193,17 +2187,16 @@ bool QOCIDriver::hasFeature(DriverFeature f) const
 
 static void qParseOpts(const QString &options, QOCIDriverPrivate *d)
 {
-    const QStringList opts(options.split(u';', Qt::SkipEmptyParts));
-    for (int i = 0; i < opts.count(); ++i) {
-        const QString tmp(opts.at(i));
+    const QVector<QStringView> opts(QStringView(options).split(u';', Qt::SkipEmptyParts));
+    for (const auto tmp : opts) {
         qsizetype idx;
         if ((idx = tmp.indexOf(u'=')) == -1) {
             qWarning("QOCIDriver::parseArgs: Invalid parameter: '%s'",
                      tmp.toLocal8Bit().constData());
             continue;
         }
-        const QString opt = tmp.left(idx);
-        const QString val = tmp.mid(idx + 1).simplified();
+        const QStringView opt = tmp.left(idx);
+        const QStringView val = tmp.mid(idx + 1).trimmed();
         bool ok;
         if (opt == "OCI_ATTR_PREFETCH_ROWS"_L1) {
             d->prefetchRows = val.toInt(&ok);
@@ -2213,9 +2206,18 @@ static void qParseOpts(const QString &options, QOCIDriverPrivate *d)
             d->prefetchMem = val.toInt(&ok);
             if (!ok)
                 d->prefetchMem = -1;
+        } else if (opt == "OCI_AUTH_MODE"_L1) {
+            if (val == "OCI_SYSDBA"_L1) {
+                d->authMode = OCI_SYSDBA;
+            } else if (val == "OCI_SYSOPER"_L1) {
+                d->authMode = OCI_SYSOPER;
+            } else if (val != "OCI_DEFAULT"_L1) {
+                qWarning("QOCIDriver::parseArgs: Unsupported value for OCI_AUTH_MODE: '%s'",
+                         val.toLocal8Bit().constData());
+            }
         } else {
-            qWarning ("QOCIDriver::parseArgs: Invalid parameter: '%s'",
-                      opt.toLocal8Bit().constData());
+            qWarning("QOCIDriver::parseArgs: Invalid parameter: '%s'",
+                     opt.toLocal8Bit().constData());
         }
     }
 }
@@ -2247,7 +2249,7 @@ bool QOCIDriver::open(const QString & db,
     if (r == OCI_SUCCESS) {
         r = OCIServerAttach(d->srvhp, d->err,
                             reinterpret_cast<const OraText *>(connectionString.utf16()),
-                            connectionString.length() * sizeof(QChar), OCI_DEFAULT);
+                            sb4(connectionString.length() * sizeof(QChar)), OCI_DEFAULT);
     }
     Q_ASSERT(!d->svc);
     if (r == OCI_SUCCESS || r == OCI_SUCCESS_WITH_INFO) {
@@ -2263,11 +2265,11 @@ bool QOCIDriver::open(const QString & db,
     }
     if (r == OCI_SUCCESS) {
         r = OCIAttrSet(d->authp, OCI_HTYPE_SESSION, const_cast<ushort *>(user.utf16()),
-                       user.length() * sizeof(QChar), OCI_ATTR_USERNAME, d->err);
+                       ub4(user.length() * sizeof(QChar)), OCI_ATTR_USERNAME, d->err);
     }
     if (r == OCI_SUCCESS) {
         r = OCIAttrSet(d->authp, OCI_HTYPE_SESSION, const_cast<ushort *>(password.utf16()),
-                       password.length() * sizeof(QChar), OCI_ATTR_PASSWORD, d->err);
+                       ub4(password.length() * sizeof(QChar)), OCI_ATTR_PASSWORD, d->err);
     }
     Q_ASSERT(!d->trans);
     if (r == OCI_SUCCESS) {
@@ -2279,9 +2281,9 @@ bool QOCIDriver::open(const QString & db,
 
     if (r == OCI_SUCCESS) {
         if (user.isEmpty() && password.isEmpty())
-            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_EXT, OCI_DEFAULT);
+            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_EXT, d->authMode);
         else
-            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_RDBMS, OCI_DEFAULT);
+            r = OCISessionBegin(d->svc, d->err, d->authp, OCI_CRED_RDBMS, d->authMode);
     }
     if (r == OCI_SUCCESS || r == OCI_SUCCESS_WITH_INFO)
         r = OCIAttrSet(d->svc, OCI_HTYPE_SVCCTX, d->authp, 0, OCI_ATTR_SESSION, d->err);
@@ -2682,7 +2684,7 @@ QSqlIndex QOCIDriver::primaryIndex(const QString& tablename) const
 
 QString QOCIDriver::formatValue(const QSqlField &field, bool trimStrings) const
 {
-    switch (field.typeID()) {
+    switch (field.metaType().id()) {
     case QMetaType::QDateTime: {
         QDateTime datetime = field.value().toDateTime();
         QString datestring;
@@ -2744,10 +2746,19 @@ QString QOCIDriver::escapeIdentifier(const QString &identifier, IdentifierType t
     QString res = identifier;
     if (!identifier.isEmpty() && !isIdentifierEscaped(identifier, type)) {
         res.replace(u'"', "\"\""_L1);
-        res.prepend(u'"').append(u'"');
         res.replace(u'.', "\".\""_L1);
+        res = u'"' + res + u'"';
     }
     return res;
 }
 
+int QOCIDriver::maximumIdentifierLength(IdentifierType type) const
+{
+    Q_D(const QOCIDriver);
+    Q_UNUSED(type);
+    return d->serverVersion > 12 ? 128 : 30;
+}
+
 QT_END_NAMESPACE
+
+#include "moc_qsql_oci_p.cpp"
