@@ -1,37 +1,14 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QTestEventLoop>
 #include <QSignalSpy>
 #include <QSemaphore>
 #include <QAbstractEventDispatcher>
+#if defined(Q_OS_WIN32)
 #include <QWinEventNotifier>
+#endif
 
 #include <qcoreapplication.h>
 #include <qelapsedtimer.h>
@@ -42,6 +19,8 @@
 #include <qdebug.h>
 #include <qmetaobject.h>
 #include <qscopeguard.h>
+#include <private/qobject_p.h>
+#include <private/qthread_p.h>
 
 #ifdef Q_OS_UNIX
 #include <pthread.h>
@@ -59,6 +38,8 @@
 
 #include <QtTest/private/qemulationdetector_p.h>
 
+using namespace std::chrono_literals;
+
 class tst_QThread : public QObject
 {
     Q_OBJECT
@@ -72,6 +53,7 @@ private slots:
     void setStackSize();
     void exit();
     void start();
+    void startSlotUsedInStringBasedLookups();
     void terminate();
     void quit();
     void started();
@@ -91,6 +73,7 @@ private slots:
     void adoptedThreadExecFinished();
     void adoptMultipleThreads();
     void adoptMultipleThreadsOverlap();
+    void adoptedThreadBindingStatus();
 
     void exitAndStart();
     void exitAndExec();
@@ -114,6 +97,12 @@ private slots:
     void create();
     void createDestruction();
     void threadIdReuse();
+
+    void terminateAndPrematureDestruction();
+    void terminateAndDoubleDestruction();
+    void terminateSelfStressTest();
+
+    void bindingListCleanupAfterDelete();
 };
 
 enum { one_minute = 60 * 1000, five_minutes = 5 * one_minute };
@@ -260,17 +249,19 @@ public:
         elapsed = 0;
         QElapsedTimer timer;
         timer.start();
+        std::chrono::nanoseconds dur{0};
         switch (sleepType) {
         case Second:
-            sleep(interval);
+            dur = std::chrono::seconds{interval};
             break;
         case Millisecond:
-            msleep(interval);
+            dur = std::chrono::milliseconds{interval};
             break;
         case Microsecond:
-            usleep(interval);
+            dur = std::chrono::microseconds{interval};
             break;
         }
+        sleep(dur);
         elapsed = timer.elapsed();
 
         cond.wakeOne();
@@ -477,11 +468,65 @@ void tst_QThread::start()
     }
 }
 
+class QThreadStarter : public QObject
+{
+    Q_OBJECT
+public:
+    using QObject::QObject;
+Q_SIGNALS:
+    void start(QThread::Priority);
+};
+
+class QThreadSelfStarter : public QThread
+{
+    Q_OBJECT
+public:
+    using QThread::QThread;
+
+    void check()
+    {
+        QVERIFY(connect(this, SIGNAL(starting(Priority)),
+                        this, SLOT(start(Priority))));
+        QVERIFY(QMetaObject::invokeMethod(this, "start", Q_ARG(Priority, IdlePriority)));
+    }
+
+Q_SIGNALS:
+    void starting(Priority);
+};
+
+void tst_QThread::startSlotUsedInStringBasedLookups()
+{
+    // QTBUG-124723
+
+    QThread thread;
+    {
+        QThreadStarter starter;
+        QVERIFY(QObject::connect(&starter, SIGNAL(start(QThread::Priority)),
+                                 &thread, SLOT(start(QThread::Priority))));
+    }
+    {
+        QThreadSelfStarter selfStarter;
+        selfStarter.check();
+        if (QTest::currentTestFailed())
+            return;
+        selfStarter.exit();
+        selfStarter.wait(30s);
+    }
+    QVERIFY(QMetaObject::invokeMethod(&thread, "start",
+                                      Q_ARG(QThread::Priority, QThread::IdlePriority)));
+    thread.exit();
+    thread.wait(30s);
+}
+
 void tst_QThread::terminate()
 {
 #if defined(Q_OS_ANDROID)
     QSKIP("Thread termination is not supported on Android.");
 #endif
+#if defined(__SANITIZE_ADDRESS__) || __has_feature(address_sanitizer)
+    QSKIP("Thread termination might result in stack underflow address sanitizer errors.");
+#endif
+
     Terminate_Thread thread;
     {
         QMutexLocker locker(&thread.mutex);
@@ -548,6 +593,10 @@ void tst_QThread::terminated()
 #if defined(Q_OS_ANDROID)
     QSKIP("Thread termination is not supported on Android.");
 #endif
+#if defined(__SANITIZE_ADDRESS__) || __has_feature(address_sanitizer)
+    QSKIP("Thread termination might result in stack underflow address sanitizer errors.");
+#endif
+
     SignalRecorder recorder;
     Terminate_Thread thread;
     connect(&thread, SIGNAL(finished()), &recorder, SLOT(slot()), Qt::DirectConnection);
@@ -953,6 +1002,20 @@ void tst_QThread::adoptMultipleThreadsOverlap()
     QCOMPARE(recorder.activationCount.loadRelaxed(), numThreads);
 }
 
+void tst_QThread::adoptedThreadBindingStatus()
+{
+    NativeThreadWrapper nativeThread;
+    nativeThread.setWaitForStop();
+
+    nativeThread.startAndWait();
+    QVERIFY(nativeThread.qthread);
+    auto privThread = static_cast<QThreadPrivate *>(QObjectPrivate::get(nativeThread.qthread));
+    QVERIFY(privThread->m_statusOrPendingObjects.bindingStatus());
+
+    nativeThread.stop();
+    nativeThread.join();
+}
+
 // Disconnects on WinCE
 void tst_QThread::stressTest()
 {
@@ -1218,10 +1281,6 @@ void tst_QThread::isRunningInFinished()
     }
 }
 
-QT_BEGIN_NAMESPACE
-Q_CORE_EXPORT uint qGlobalPostedEventsCount();
-QT_END_NAMESPACE
-
 class DummyEventDispatcher : public QAbstractEventDispatcher {
 public:
     DummyEventDispatcher() : QAbstractEventDispatcher() {}
@@ -1348,9 +1407,6 @@ void tst_QThread::quitLock()
 
 void tst_QThread::create()
 {
-#if !QT_CONFIG(cxx11_future)
-    QSKIP("This test requires QThread::create");
-#else
     {
         const auto &function = [](){};
         QScopedPointer<QThread> thread(QThread::create(function));
@@ -1590,7 +1646,6 @@ void tst_QThread::create()
         QVERIFY(!thread);
     }
 #endif // QT_NO_EXCEPTIONS
-#endif // QT_CONFIG(cxx11_future)
 }
 
 void tst_QThread::createDestruction()
@@ -1600,7 +1655,7 @@ void tst_QThread::createDestruction()
             for (;;) {
                 if (QThread::currentThread()->isInterruptionRequested())
                     return;
-                QThread::msleep(1);
+                QThread::sleep(1ms);
             }
         };
 
@@ -1719,7 +1774,7 @@ void tst_QThread::threadIdReuse()
     bool threadIdReused = false;
 
     for (int i = 0; i < 42; i++) {
-        QThread::msleep(1);
+        QThread::sleep(1ms);
 
         Qt::HANDLE threadId2;
         bool waitOk = false;
@@ -1746,5 +1801,147 @@ void tst_QThread::threadIdReuse()
     }
 }
 
+class WaitToRun_Thread : public QThread
+{
+    Q_OBJECT
+public:
+    void run() override
+    {
+        emit running();
+        QThread::exec();
+    }
+
+Q_SIGNALS:
+    void running();
+};
+
+
+void tst_QThread::terminateAndPrematureDestruction()
+{
+#if defined(__SANITIZE_ADDRESS__) || __has_feature(address_sanitizer)
+    QSKIP("Thread termination might result in stack underflow address sanitizer errors.");
+#endif
+
+    WaitToRun_Thread thread;
+    QSignalSpy spy(&thread, &WaitToRun_Thread::running);
+    thread.start();
+    QVERIFY(spy.wait(500));
+
+    QScopedPointer<QObject> obj(new QObject);
+    QPointer<QObject> pObj(obj.data());
+    obj->deleteLater();
+
+    thread.terminate();
+    QVERIFY2(pObj, "object was deleted prematurely!");
+    thread.wait(500);
+}
+
+void tst_QThread::terminateAndDoubleDestruction()
+{
+#if defined(__SANITIZE_ADDRESS__) || __has_feature(address_sanitizer)
+    QSKIP("Thread termination might result in stack underflow address sanitizer errors.");
+#endif
+
+    class ChildObject : public QObject
+    {
+    public:
+        ChildObject(QObject *parent)
+            : QObject(parent)
+        {
+            QSignalSpy spy(&thread, &WaitToRun_Thread::running);
+            thread.start();
+            spy.wait(500);
+        }
+
+        ~ChildObject()
+        {
+            QVERIFY2(!inDestruction, "Double object destruction!");
+            inDestruction = true;
+            thread.terminate();
+            thread.wait(500);
+        }
+
+        bool inDestruction = false;
+        WaitToRun_Thread thread;
+    };
+
+    class TestObject : public QObject
+    {
+    public:
+        TestObject()
+            : child(new ChildObject(this))
+        {
+        }
+
+        ~TestObject()
+        {
+            child->deleteLater();
+        }
+
+        ChildObject *child = nullptr;
+    };
+
+    TestObject obj;
+}
+
+void tst_QThread::terminateSelfStressTest()
+{
+    // This simply tests that QThread::terminate() doesn't crash or causes
+    // sanitizer reports when a thread cancels itself.
+#ifdef Q_OS_ANDROID
+    QSKIP("Android cannot cancel threads");
+#endif
+
+#ifdef Q_OS_WIN
+    QSKIP("QTBUG-127050");
+#endif
+
+    struct Thread : QThread {
+        void run() override {
+            terminate();
+            while (true) sleep(1ns); // QTBUG-127008
+        }
+    };
+
+    {
+        // first, try with one:
+        Thread t;
+        t.start();
+        QVERIFY(t.wait(10s));
+    }
+
+    constexpr QThread::Priority priorities[] = {
+        QThread::IdlePriority,
+        QThread::LowestPriority,
+        QThread::LowPriority,
+        QThread::NormalPriority,
+        QThread::HighPriority,
+        QThread::HighestPriority,
+        QThread::TimeCriticalPriority,
+        QThread::InheritPriority,
+    };
+
+    QVarLengthArray<Thread, 1024> threads(3 * QThread::idealThreadCount());
+
+    size_t i = 0;
+    for (Thread &t : threads)
+        t.start(priorities[i++ % std::size(priorities)]);
+
+    for (Thread &t : threads)
+        QVERIFY2(t.wait(60s), QByteArray::number(&t - threads.data()).constData());
+}
+
+void tst_QThread::bindingListCleanupAfterDelete()
+{
+    QThread t;
+    auto optr = std::make_unique<QObject>();
+    optr->moveToThread(&t);
+    auto threadPriv =  static_cast<QThreadPrivate *>(QObjectPrivate::get(&t));
+    auto list = threadPriv->m_statusOrPendingObjects.list();
+    QVERIFY(list);
+    optr.reset();
+    QVERIFY(list->empty());
+}
+
 QTEST_MAIN(tst_QThread)
 #include "tst_qthread.moc"
