@@ -1,31 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
-#include <QtTest/QtTest>
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
+#include <QTest>
 #include <QtXml/QtXml>
 #include <QtGui/QFontInfo>
 #include <QtGui/QFontMetrics>
@@ -74,10 +50,16 @@ private slots:
     void gradient();
     void extractFontFamily_data();
     void extractFontFamily();
+    void extractFontSize_data();
+    void extractFontSize();
     void extractBorder_data();
     void extractBorder();
     void noTextDecoration();
     void quotedAndUnquotedIdentifiers();
+    void whitespaceValues_data();
+    void whitespaceValues();
+    void borderColor_data();
+    void borderColor();
 };
 
 void tst_QCssParser::scanner_data()
@@ -85,14 +67,15 @@ void tst_QCssParser::scanner_data()
     QTest::addColumn<QString>("input");
     QTest::addColumn<QString>("output");
 
-#if defined(Q_OS_ANDROID) || defined(Q_OS_WINRT)
+#if defined(Q_OS_ANDROID)
     QDir d(":/");
 #else
-    QDir d(SRCDIR);
+    QDir d(QT_TESTCASE_SOURCEDIR);
 #endif
     d.cd("testdata");
     d.cd("scanner");
-    foreach (QFileInfo test, d.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot)) {
+    const auto entries = d.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
+    for (const QFileInfo &test : entries) {
         QString dir = test.absoluteFilePath() + QDir::separator();
         QTest::newRow(qPrintable(test.baseName()))
             << dir + "input"
@@ -146,10 +129,10 @@ static const char *tokenName(QCss::TokenType t)
     return "";
 }
 
-static void debug(const QVector<QCss::Symbol> &symbols, int index = -1)
+static void debug(const QList<QCss::Symbol> &symbols, int index = -1)
 {
     qDebug() << "all symbols:";
-    for (int i = 0; i < symbols.count(); ++i)
+    for (int i = 0; i < symbols.size(); ++i)
         qDebug() << '(' << i << "); Token:" << tokenName(symbols.at(i).token) << "; Lexem:" << symbols.at(i).lexem();
     if (index != -1)
         qDebug() << "failure at index" << index;
@@ -164,13 +147,13 @@ void tst_QCssParser::scanner()
 
     QFile inputFile(input);
     QVERIFY(inputFile.open(QIODevice::ReadOnly|QIODevice::Text));
-    QVector<QCss::Symbol> symbols;
+    QList<QCss::Symbol> symbols;
     QCss::Scanner::scan(QCss::Scanner::preprocess(QString::fromUtf8(inputFile.readAll())), &symbols);
 
-    QVERIFY(symbols.count() > 1);
+    QVERIFY(symbols.size() > 1);
     QCOMPARE(symbols.last().token, QCss::S);
     QCOMPARE(symbols.last().lexem(), QLatin1String("\n"));
-    symbols.remove(symbols.count() - 1, 1);
+    symbols.remove(symbols.size() - 1, 1);
 
     QFile outputFile(output);
     QVERIFY(outputFile.open(QIODevice::ReadOnly|QIODevice::Text));
@@ -182,14 +165,14 @@ void tst_QCssParser::scanner()
         lines.append(line);
     }
 
-    if (lines.count() != symbols.count()) {
+    if (lines.size() != symbols.size()) {
         debug(symbols);
-        QCOMPARE(lines.count(), symbols.count());
+        QCOMPARE(lines.size(), symbols.size());
     }
 
-    for (int i = 0; i < lines.count(); ++i) {
+    for (int i = 0; i < lines.size(); ++i) {
         QStringList l = lines.at(i).split(QChar::fromLatin1('|'));
-        QCOMPARE(l.count(), 2);
+        QCOMPARE(l.size(), 2);
         const QString expectedToken = l.at(0);
         const QString expectedLexem = l.at(1);
         QString actualToken = QString::fromLatin1(tokenName(symbols.at(i).token));
@@ -341,9 +324,9 @@ void tst_QCssParser::expr_data()
 {
     QTest::addColumn<bool>("parseSuccess");
     QTest::addColumn<QString>("css");
-    QTest::addColumn<QVector<QCss::Value> >("expectedValues");
+    QTest::addColumn<QList<QCss::Value>>("expectedValues");
 
-    QVector<QCss::Value> values;
+    QList<QCss::Value> values;
     QCss::Value val;
 
     QCss::Value comma;
@@ -366,16 +349,16 @@ void tst_QCssParser::expr()
 {
     QFETCH(bool, parseSuccess);
     QFETCH(QString, css);
-    QFETCH(QVector<QCss::Value>, expectedValues);
+    QFETCH(QList<QCss::Value>, expectedValues);
 
     QCss::Parser parser(css);
-    QVector<QCss::Value> values;
+    QList<QCss::Value> values;
     QVERIFY(parser.testExpr());
     QCOMPARE(parser.parseExpr(&values), parseSuccess);
     if (parseSuccess) {
-        QCOMPARE(values.count(), expectedValues.count());
+        QCOMPARE(values.size(), expectedValues.size());
 
-        for (int i = 0; i < values.count(); ++i) {
+        for (int i = 0; i < values.size(); ++i) {
             QCOMPARE(int(values.at(i).type), int(expectedValues.at(i).type));
             QCOMPARE(values.at(i).variant, expectedValues.at(i).variant);
         }
@@ -394,7 +377,7 @@ void tst_QCssParser::import()
     QVERIFY(parser.testImport());
     QVERIFY(parser.parseImport(&rule));
     QCOMPARE(rule.href, QString("www.kde.org"));
-    QCOMPARE(rule.media.count(), 2);
+    QCOMPARE(rule.media.size(), 2);
     QCOMPARE(rule.media.at(0), QString("print"));
     QCOMPARE(rule.media.at(1), QString("screen"));
 }
@@ -405,7 +388,7 @@ void tst_QCssParser::media()
     QVERIFY(parser.testMedia());
     QCss::MediaRule rule;
     QVERIFY(parser.parseMedia(&rule));
-    QCOMPARE(rule.media.count(), 2);
+    QCOMPARE(rule.media.size(), 2);
     QCOMPARE(rule.media.at(0), QString("print"));
     QCOMPARE(rule.media.at(1), QString("screen"));
     QVERIFY(rule.styleRules.isEmpty());
@@ -428,8 +411,8 @@ void tst_QCssParser::ruleset()
         QVERIFY(parser.testRuleset());
         QCss::StyleRule rule;
         QVERIFY(parser.parseRuleset(&rule));
-        QCOMPARE(rule.selectors.count(), 1);
-        QCOMPARE(rule.selectors.at(0).basicSelectors.count(), 1);
+        QCOMPARE(rule.selectors.size(), 1);
+        QCOMPARE(rule.selectors.at(0).basicSelectors.size(), 1);
         QCOMPARE(rule.selectors.at(0).basicSelectors.at(0).elementName, QString("p"));
         QVERIFY(rule.declarations.isEmpty());
     }
@@ -439,10 +422,10 @@ void tst_QCssParser::ruleset()
         QVERIFY(parser.testRuleset());
         QCss::StyleRule rule;
         QVERIFY(parser.parseRuleset(&rule));
-        QCOMPARE(rule.selectors.count(), 2);
-        QCOMPARE(rule.selectors.at(0).basicSelectors.count(), 1);
+        QCOMPARE(rule.selectors.size(), 2);
+        QCOMPARE(rule.selectors.at(0).basicSelectors.size(), 1);
         QCOMPARE(rule.selectors.at(0).basicSelectors.at(0).elementName, QString("p"));
-        QCOMPARE(rule.selectors.at(1).basicSelectors.count(), 1);
+        QCOMPARE(rule.selectors.at(1).basicSelectors.size(), 1);
         QCOMPARE(rule.selectors.at(1).basicSelectors.at(0).elementName, QString("div"));
         QVERIFY(rule.declarations.isEmpty());
     }
@@ -452,14 +435,14 @@ void tst_QCssParser::ruleset()
         QVERIFY(parser.testRuleset());
         QCss::StyleRule rule;
         QVERIFY(parser.parseRuleset(&rule));
-        QCOMPARE(rule.selectors.count(), 2);
+        QCOMPARE(rule.selectors.size(), 2);
 
-        QCOMPARE(rule.selectors.at(0).basicSelectors.count(), 1);
-        QCOMPARE(rule.selectors.at(0).basicSelectors.at(0).pseudos.count(), 1);
+        QCOMPARE(rule.selectors.at(0).basicSelectors.size(), 1);
+        QCOMPARE(rule.selectors.at(0).basicSelectors.at(0).pseudos.size(), 1);
         QCOMPARE(rule.selectors.at(0).basicSelectors.at(0).pseudos.at(0).name, QString("before"));
 
-        QCOMPARE(rule.selectors.at(1).basicSelectors.count(), 1);
-        QCOMPARE(rule.selectors.at(1).basicSelectors.at(0).pseudos.count(), 1);
+        QCOMPARE(rule.selectors.at(1).basicSelectors.size(), 1);
+        QCOMPARE(rule.selectors.at(1).basicSelectors.at(0).pseudos.size(), 1);
         QCOMPARE(rule.selectors.at(1).basicSelectors.at(0).pseudos.at(0).name, QString("after"));
 
         QVERIFY(rule.declarations.isEmpty());
@@ -711,21 +694,21 @@ void tst_QCssParser::selector()
     QCss::Selector selector;
     QVERIFY(parser.parseSelector(&selector));
 
-    QCOMPARE(selector.basicSelectors.count(), expectedSelector.basicSelectors.count());
-    for (int i = 0; i < selector.basicSelectors.count(); ++i) {
+    QCOMPARE(selector.basicSelectors.size(), expectedSelector.basicSelectors.size());
+    for (int i = 0; i < selector.basicSelectors.size(); ++i) {
         const QCss::BasicSelector sel = selector.basicSelectors.at(i);
         const QCss::BasicSelector expectedSel = expectedSelector.basicSelectors.at(i);
         QCOMPARE(sel.elementName, expectedSel.elementName);
         QCOMPARE(int(sel.relationToNext), int(expectedSel.relationToNext));
 
-        QCOMPARE(sel.pseudos.count(), expectedSel.pseudos.count());
-        for (int i = 0; i < sel.pseudos.count(); ++i) {
+        QCOMPARE(sel.pseudos.size(), expectedSel.pseudos.size());
+        for (int i = 0; i < sel.pseudos.size(); ++i) {
             QCOMPARE(sel.pseudos.at(i).name, expectedSel.pseudos.at(i).name);
             QCOMPARE(sel.pseudos.at(i).function, expectedSel.pseudos.at(i).function);
         }
 
-        QCOMPARE(sel.attributeSelectors.count(), expectedSel.attributeSelectors.count());
-        for (int i = 0; i < sel.attributeSelectors.count(); ++i) {
+        QCOMPARE(sel.attributeSelectors.size(), expectedSel.attributeSelectors.size());
+        for (int i = 0; i < sel.attributeSelectors.size(); ++i) {
             QCOMPARE(sel.attributeSelectors.at(i).name, expectedSel.attributeSelectors.at(i).name);
             QCOMPARE(sel.attributeSelectors.at(i).value, expectedSel.attributeSelectors.at(i).value);
             QCOMPARE(int(sel.attributeSelectors.at(i).valueMatchCriterium), int(expectedSel.attributeSelectors.at(i).valueMatchCriterium));
@@ -783,13 +766,13 @@ void tst_QCssParser::malformedDeclarations()
     QCss::StyleRule rule;
     QVERIFY(parser.parseRuleset(&rule));
 
-    QCOMPARE(rule.selectors.count(), 1);
-    QCOMPARE(rule.selectors.at(0).basicSelectors.count(), 1);
+    QCOMPARE(rule.selectors.size(), 1);
+    QCOMPARE(rule.selectors.at(0).basicSelectors.size(), 1);
     QCOMPARE(rule.selectors.at(0).basicSelectors.at(0).elementName, QString("p"));
 
-    QVERIFY(rule.declarations.count() >= 1);
+    QVERIFY(rule.declarations.size() >= 1);
     QCOMPARE(int(rule.declarations.last().d->propertyId), int(QCss::Color));
-    QCOMPARE(rule.declarations.last().d->values.count(), 1);
+    QCOMPARE(rule.declarations.last().d->values.size(), 1);
     QCOMPARE(int(rule.declarations.last().d->values.at(0).type), int(QCss::Value::Identifier));
     QCOMPARE(rule.declarations.last().d->values.at(0).variant.toString(), QString("green"));
 }
@@ -809,17 +792,17 @@ void tst_QCssParser::invalidAtKeywords()
     QCss::StyleSheet sheet;
     QVERIFY(parser.parse(&sheet));
 
-    QCOMPARE(sheet.styleRules.count() + sheet.nameIndex.count(), 1);
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size(), 1);
     QCss::StyleRule rule =  (!sheet.styleRules.isEmpty()) ?
             sheet.styleRules.at(0) : *sheet.nameIndex.begin();
 
-    QCOMPARE(rule.selectors.count(), 1);
-    QCOMPARE(rule.selectors.at(0).basicSelectors.count(), 1);
+    QCOMPARE(rule.selectors.size(), 1);
+    QCOMPARE(rule.selectors.at(0).basicSelectors.size(), 1);
     QCOMPARE(rule.selectors.at(0).basicSelectors.at(0).elementName, QString("h1"));
 
-    QCOMPARE(rule.declarations.count(), 1);
+    QCOMPARE(rule.declarations.size(), 1);
     QCOMPARE(int(rule.declarations.at(0).d->propertyId), int(QCss::Color));
-    QCOMPARE(rule.declarations.at(0).d->values.count(), 1);
+    QCOMPARE(rule.declarations.at(0).d->values.size(), 1);
     QCOMPARE(int(rule.declarations.at(0).d->values.at(0).type), int(QCss::Value::Identifier));
     QCOMPARE(rule.declarations.at(0).d->values.at(0).variant.toString(), QString("blue"));
 }
@@ -835,17 +818,31 @@ void tst_QCssParser::colorValue_data()
     QTest::newRow("hexcolor") << "color: #12af0e" << QColor(0x12, 0xaf, 0x0e);
     QTest::newRow("functional1") << "color: rgb(21, 45, 73)" << QColor(21, 45, 73);
     QTest::newRow("functional2") << "color: rgb(100%, 0%, 100%)" << QColor(0xff, 0, 0xff);
+    QTest::newRow("rgb") << "color: rgb(10, 20, 30)" << QColor(10, 20, 30);
     QTest::newRow("rgba") << "color: rgba(10, 20, 30, 40)" << QColor(10, 20, 30, 40);
     QTest::newRow("rgbaf") << "color: rgba(10, 20, 30, 0.5)" << QColor(10, 20, 30, 127);
-    QTest::newRow("rgb") << "color: rgb(10, 20, 30, 40)" << QColor(10, 20, 30, 40);
-    QTest::newRow("hsl") << "color: hsv(10, 20, 30)" << QColor::fromHsv(10, 20, 30, 255);
-    QTest::newRow("hsla") << "color: hsva(10, 20, 30, 40)" << QColor::fromHsv(10, 20, 30, 40);
+    QTest::newRow("hsv") << "color: hsv(10, 20, 30)" << QColor::fromHsv(10, 20, 30);
+    QTest::newRow("hsva") << "color: hsva(10, 20, 30, 40)" << QColor::fromHsv(10, 20, 30, 40);
+    // the percent and float values are well chosen to not get in trouble due to rounding errors
+    QTest::newRow("hsva-percent") << "color: hsva(100%, 20%, 40%, 60%)" << QColor::fromHsv(359, 51, 102, 153);
+    QTest::newRow("hsva-float") << "color: hsva(180, 20%, 40%, 0.6)" << QColor::fromHsvF(0.5f, 0.2f, 0.4f, 0.6f);
+    QTest::newRow("hsl") << "color: hsl(60, 100%, 50%)" << QColor::fromHsl(60., 255, 127);
+    QTest::newRow("hsla") << "color: hsla(240, 255, 127, 192)" << QColor::fromHsl(240, 255, 127, 192);
+    QTest::newRow("hsla-percent") << "color: hsla(100%, 80%, 40%, 0%)" << QColor::fromHsl(359, 204, 102, 0);
+    QTest::newRow("hsla-float") << "color: hsla(252, 40%, 60%, 0.2)" << QColor::fromHslF(0.7f, 0.4f, 0.6f, 0.2f);
     QTest::newRow("invalid1") << "color: rgb(why, does, it, always, rain, on, me)" << QColor();
     QTest::newRow("invalid2") << "color: rgba(i, meant, norway)" << QColor();
     QTest::newRow("invalid3") << "color: rgb(21)" << QColor();
+    QTest::newRow("invalid4") << "color: rgbx(1, 2, 3)" << QColor();
+    QTest::newRow("invalid5") << "color: rgbax(1, 2, 3, 4)" << QColor();
+    QTest::newRow("invalid6") << "color: hsv(360, 0, 0)" << QColor();
+    QTest::newRow("invalid7") << "color: hsla(1, a, 1, 21)" << QColor();
     QTest::newRow("role") << "color: palette(base)" << qApp->palette().color(QPalette::Base);
     QTest::newRow("role2") << "color: palette( window-text ) " << qApp->palette().color(QPalette::WindowText);
     QTest::newRow("transparent") << "color: transparent" << QColor(Qt::transparent);
+
+    QTest::newRow("rgb-invalid") << "color: rgb(10, 20, 30, 40)" << QColor();
+    QTest::newRow("rgba-invalid") << "color: rgba(10, 20, 30)" << QColor();
 }
 
 void tst_QCssParser::colorValue()
@@ -870,32 +867,34 @@ public:
         styleSheets.append(sheet);
     }
 
-    virtual QStringList nodeNames(NodePtr node) const { return QStringList(reinterpret_cast<QDomElement *>(node.ptr)->tagName()); }
-    virtual QString attribute(NodePtr node, const QString &name) const { return reinterpret_cast<QDomElement *>(node.ptr)->attribute(name); }
-    virtual bool hasAttribute(NodePtr node, const QString &name) const { return reinterpret_cast<QDomElement *>(node.ptr)->hasAttribute(name); }
-    virtual bool hasAttributes(NodePtr node) const { return reinterpret_cast<QDomElement *>(node.ptr)->hasAttributes(); }
+    QStringList nodeNames(NodePtr node) const override
+    { return QStringList(reinterpret_cast<QDomElement *>(node.ptr)->tagName()); }
+    QString attributeValue(NodePtr node, const QCss::AttributeSelector &aSel) const override
+    { return reinterpret_cast<QDomElement *>(node.ptr)->attribute(aSel.name); }
+    bool hasAttribute(NodePtr node, const QString &name) const
+    { return reinterpret_cast<QDomElement *>(node.ptr)->hasAttribute(name); }
+    bool hasAttributes(NodePtr node) const override
+    { return reinterpret_cast<QDomElement *>(node.ptr)->hasAttributes(); }
 
-    virtual bool isNullNode(NodePtr node) const {
-        return reinterpret_cast<QDomElement *>(node.ptr)->isNull();
-    }
-    virtual NodePtr parentNode(NodePtr node) const {
+    bool isNullNode(NodePtr node) const override
+    { return reinterpret_cast<QDomElement *>(node.ptr)->isNull(); }
+    NodePtr parentNode(NodePtr node) const override {
         NodePtr parent;
         parent.ptr = new QDomElement(reinterpret_cast<QDomElement *>(node.ptr)->parentNode().toElement());
         return parent;
     }
-    virtual NodePtr duplicateNode(NodePtr node) const {
+    NodePtr duplicateNode(NodePtr node) const override {
         NodePtr n;
         n.ptr = new QDomElement(*reinterpret_cast<QDomElement *>(node.ptr));
         return n;
     }
-    virtual NodePtr previousSiblingNode(NodePtr node) const {
+    NodePtr previousSiblingNode(NodePtr node) const override {
         NodePtr sibling;
         sibling.ptr = new QDomElement(reinterpret_cast<QDomElement *>(node.ptr)->previousSiblingElement());
         return sibling;
     }
-    virtual void freeNode(NodePtr node) const {
-        delete reinterpret_cast<QDomElement *>(node.ptr);
-    }
+    void freeNode(NodePtr node) const override
+    { delete reinterpret_cast<QDomElement *>(node.ptr); }
 
 private:
     QDomDocument doc;
@@ -966,8 +965,8 @@ void tst_QCssParser::marginValue()
     QDomElement e = doc.documentElement().firstChildElement();
     QCss::StyleSelector::NodePtr n;
     n.ptr = &e;
-    QVector<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
-    QVector<QCss::Declaration> decls = rules.at(0).declarations;
+    QList<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
+    QList<QCss::Declaration> decls = rules.at(0).declarations;
     QCss::ValueExtractor v(decls);
 
     {
@@ -1145,12 +1144,12 @@ void tst_QCssParser::styleSelector()
     QVERIFY(!e.isNull());
     QCss::StyleSelector::NodePtr n;
     n.ptr = &e;
-    QVector<QCss::Declaration> decls = testSelector.declarationsForNode(n);
+    QList<QCss::Declaration> decls = testSelector.declarationsForNode(n);
 
     if (match) {
-        QCOMPARE(decls.count(), 1);
+        QCOMPARE(decls.size(), 1);
         QCOMPARE(int(decls.at(0).d->propertyId), int(QCss::BackgroundColor));
-        QCOMPARE(decls.at(0).d->values.count(), 1);
+        QCOMPARE(decls.at(0).d->values.size(), 1);
         QCOMPARE(int(decls.at(0).d->values.at(0).type), int(QCss::Value::Identifier));
         QCOMPARE(decls.at(0).d->values.at(0).variant.toString(), QString("green"));
     } else {
@@ -1190,11 +1189,11 @@ void tst_QCssParser::specificity()
     QCss::StyleSheet sheet;
     QVERIFY(parser.parse(&sheet));
 
-    QCOMPARE(sheet.styleRules.count() + sheet.nameIndex.count() + sheet.idIndex.count() , 1);
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size() + sheet.idIndex.size() , 1);
     QCss::StyleRule rule =  (!sheet.styleRules.isEmpty()) ? sheet.styleRules.at(0)
                         :  (!sheet.nameIndex.isEmpty())  ? *sheet.nameIndex.begin()
                         :  *sheet.idIndex.begin();
-    QCOMPARE(rule.selectors.count(), 1);
+    QCOMPARE(rule.selectors.size(), 1);
     QTEST(rule.selectors.at(0).specificity(), "specificity");
 }
 
@@ -1241,17 +1240,17 @@ void tst_QCssParser::specificitySort()
         QDomElement e = doc.documentElement().firstChildElement();
         QCss::StyleSelector::NodePtr n;
         n.ptr = &e;
-        QVector<QCss::Declaration> decls = testSelector.declarationsForNode(n);
+        QList<QCss::Declaration> decls = testSelector.declarationsForNode(n);
 
-        QCOMPARE(decls.count(), 2);
+        QCOMPARE(decls.size(), 2);
 
         QCOMPARE(int(decls.at(0).d->propertyId), int(QCss::Color));
-        QCOMPARE(decls.at(0).d->values.count(), 1);
+        QCOMPARE(decls.at(0).d->values.size(), 1);
         QCOMPARE(int(decls.at(0).d->values.at(0).type), int(QCss::Value::Identifier));
         QCOMPARE(decls.at(0).d->values.at(0).variant.toString(), QString("green"));
 
         QCOMPARE(int(decls.at(1).d->propertyId), int(QCss::Color));
-        QCOMPARE(decls.at(1).d->values.count(), 1);
+        QCOMPARE(decls.at(1).d->values.size(), 1);
         QCOMPARE(int(decls.at(1).d->values.at(0).type), int(QCss::Value::Identifier));
         QCOMPARE(decls.at(1).d->values.at(0).variant.toString(), QString("red"));
     }
@@ -1323,10 +1322,10 @@ void tst_QCssParser::rulesForNode()
     QDomElement e = doc.documentElement().firstChildElement();
     QCss::StyleSelector::NodePtr n;
     n.ptr = &e;
-    QVector<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
+    QList<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
 
-    QVector<QCss::Declaration> decls;
-    for (int i = 0; i < rules.count(); i++) {
+    QList<QCss::Declaration> decls;
+    for (int i = 0; i < rules.size(); i++) {
         const QCss::Selector &selector = rules.at(i).selectors.at(0);
         quint64 negated = 0;
         quint64 cssClass = selector.pseudoClass(&negated);
@@ -1335,7 +1334,7 @@ void tst_QCssParser::rulesForNode()
             decls += rules.at(i).declarations;
     }
 
-    QCOMPARE(decls.count(), declCount);
+    QCOMPARE(decls.size(), declCount);
 
     if (declCount > 0)
         QCOMPARE(decls.at(0).d->values.at(0).variant.toString(), value0);
@@ -1381,8 +1380,8 @@ void tst_QCssParser::shorthandBackgroundProperty()
     QDomElement e = doc.documentElement().firstChildElement();
     QCss::StyleSelector::NodePtr n;
     n.ptr = &e;
-    QVector<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
-    QVector<QCss::Declaration> decls = rules.at(0).declarations;
+    QList<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
+    QList<QCss::Declaration> decls = rules.at(0).declarations;
     QCss::ValueExtractor v(decls);
 
     QBrush brush;
@@ -1460,16 +1459,16 @@ void tst_QCssParser::pseudoElement()
     QDomElement e = doc.documentElement().firstChildElement();
     QCss::StyleSelector::NodePtr n;
     n.ptr = &e;
-    QVector<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
-    QVector<QCss::Declaration> decls;
-    for (int i = 0; i < rules.count(); i++) {
+    QList<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
+    QList<QCss::Declaration> decls;
+    for (int i = 0; i < rules.size(); i++) {
         const QCss::Selector& selector = rules.at(i).selectors.at(0);
         if (pseudoElement.compare(selector.pseudoElement(), Qt::CaseInsensitive) != 0)
             continue;
         decls += rules.at(i).declarations;
 
     }
-    QCOMPARE(decls.count(), declCount);
+    QCOMPARE(decls.size(), declCount);
 }
 
 void tst_QCssParser::gradient_data()
@@ -1509,6 +1508,12 @@ void tst_QCssParser::gradient_data()
          "spread: repeat, stop:0.2 rgb(1, 2, 3), stop:0.5 rgba(1, 2, 3, 4))" << "conical" << QPointF(4, 2) << QPointF()
                              << 2 << qreal(0.2) << QColor(1, 2, 3) << qreal(0.5) << QColor(1, 2, 3, 4);
 
+    // spaces before first function parameter lead to parser errors
+    QTest::newRow("QTBUG-61795") <<
+     "selection-background-color: qconicalgradient( cx: 4, cy : 2, angle: 23, "
+         "spread: repeat, stop:0.2 rgb( 1, 2, 3), stop:0.5 rgba( 1, 2, 3, 4))" << "conical" << QPointF(4, 2) << QPointF()
+                             << 2 << qreal(0.2) << QColor(1, 2, 3) << qreal(0.5) << QColor(1, 2, 3, 4);
+
     /* won't pass: stop values are expected to be sorted
      QTest::newRow("unsorted-stop") <<
      "selection-background: lineargradient(x1:0, y1:0, x2:0, y2:1, "
@@ -1541,23 +1546,29 @@ void tst_QCssParser::gradient()
     QDomElement e = doc.documentElement().firstChildElement();
     QCss::StyleSelector::NodePtr n;
     n.ptr = &e;
-    QVector<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
-    QVector<QCss::Declaration> decls = rules.at(0).declarations;
+    QList<QCss::StyleRule> rules = testSelector.styleRulesForNode(n);
+    QList<QCss::Declaration> decls = rules.at(0).declarations;
     QCss::ValueExtractor ve(decls);
-    QBrush fg, sfg;
-    QBrush sbg, abg;
-    QVERIFY(ve.extractPalette(&fg, &sfg, &sbg, &abg));
+    QBrush foreground;
+    QBrush selectedForeground;
+    QBrush selectedBackground;
+    QBrush alternateBackground;
+    QBrush placeHolderTextForeground;
+    QBrush accent;
+    QVERIFY(ve.extractPalette(&foreground, &selectedForeground, &selectedBackground,
+                              &alternateBackground, &placeHolderTextForeground, &accent));
+
     if (type == "linear") {
-        QCOMPARE(sbg.style(), Qt::LinearGradientPattern);
-        const QLinearGradient *lg = static_cast<const QLinearGradient *>(sbg.gradient());
+        QCOMPARE(selectedBackground.style(), Qt::LinearGradientPattern);
+        const auto *lg = static_cast<const QLinearGradient *>(selectedBackground.gradient());
         QCOMPARE(lg->start(), start);
         QCOMPARE(lg->finalStop(), finalStop);
     } else if (type == "conical") {
-        QCOMPARE(sbg.style(), Qt::ConicalGradientPattern);
-        const QConicalGradient *cg = static_cast<const QConicalGradient *>(sbg.gradient());
+        QCOMPARE(selectedBackground.style(), Qt::ConicalGradientPattern);
+        const auto *cg = static_cast<const QConicalGradient *>(selectedBackground.gradient());
         QCOMPARE(cg->center(), start);
     }
-    const QGradient *g = sbg.gradient();
+    const QGradient *g = selectedBackground.gradient();
     QCOMPARE(g->spread(), QGradient::Spread(spread));
     QCOMPARE(g->stops().at(0).first, stop0);
     QCOMPARE(g->stops().at(0).second, color0);
@@ -1597,11 +1608,11 @@ void tst_QCssParser::extractFontFamily()
     QCss::StyleSheet sheet;
     QVERIFY(parser.parse(&sheet));
 
-    QCOMPARE(sheet.styleRules.count() + sheet.nameIndex.count(), 1);
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size(), 1);
     QCss::StyleRule rule =  (!sheet.styleRules.isEmpty()) ?
             sheet.styleRules.at(0) : *sheet.nameIndex.begin();
 
-    const QVector<QCss::Declaration> decls = rule.declarations;
+    const QList<QCss::Declaration> decls = rule.declarations;
     QVERIFY(!decls.isEmpty());
     QCss::ValueExtractor extractor(decls);
 
@@ -1613,6 +1624,65 @@ void tst_QCssParser::extractFontFamily()
     QTEST(info.family(), "expectedFamily");
 }
 
+void tst_QCssParser::extractFontSize_data()
+{
+    QTest::addColumn<QString>("css");
+    QTest::addColumn<int>("expectedPixelSize");
+    QTest::addColumn<int>("expectedPointSize");
+    QTest::addColumn<qreal>("expectedPointSizeF");
+
+    QTest::newRow("integer point size") << "font-size: 12pt" << -1 << 12 << 12.0;
+    QTest::newRow("float point size round down") << "font-size: 12.3pt" << -1 << 12 << 12.3;
+    QTest::newRow("float point size midpoint") << "font-size: 12.5pt" << -1 << 13 << 12.5;
+    QTest::newRow("float point size round up") << "font-size: 12.7pt" << -1 << 13 << 12.7;
+
+    QTest::newRow("integer pixel size") << "font-size: 12px" << 12 << -1 << -1.0;
+    QTest::newRow("float pixel size round down") << "font-size: 12.3px" << 12 << -1 << -1.0;
+    QTest::newRow("float pixel size midpoint") << "font-size: 12.5px" << 13 << -1 << -1.0;
+    QTest::newRow("float pixel size round up") << "font-size: 12.7px" << 13 << -1 << -1.0;
+
+    QTest::newRow("shorthand integer point size") << "font: 12pt Arial" << -1 << 12 << 12.0;
+    QTest::newRow("shorthand float point size round down") << "font: 12.3pt Arial" << -1 << 12 << 12.3;
+    QTest::newRow("shorthand float point size midpoint") << "font: 12.5pt Arial" << -1 << 13 << 12.5;
+    QTest::newRow("shorthand float point size round up") << "font: 12.7pt Arial" << -1 << 13 << 12.7;
+
+    QTest::newRow("shorthand integer pixel size") << "font: 12px Arial" << 12 << -1 << -1.0;
+    QTest::newRow("shorthand float pixel size round down") << "font: 12.3px Arial" << 12 << -1 << -1.0;
+    QTest::newRow("shorthand float pixel size midpoint") << "font: 12.5px Arial" << 13 << -1 << -1.0;
+    QTest::newRow("shorthand float pixel size round up") << "font: 12.7px Arial" << 13 << -1 << -1.0;
+}
+
+void tst_QCssParser::extractFontSize()
+{
+    QFETCH(QString, css);
+    css.prepend("dummy {");
+    css.append(QLatin1Char('}'));
+
+    QCss::Parser parser(css);
+    QCss::StyleSheet sheet;
+    QVERIFY(parser.parse(&sheet));
+
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size(), 1);
+    QCss::StyleRule rule = (!sheet.styleRules.isEmpty()) ?
+            sheet.styleRules.at(0) : *sheet.nameIndex.begin();
+
+    const QList<QCss::Declaration> decls = rule.declarations;
+    QVERIFY(!decls.isEmpty());
+    QCss::ValueExtractor extractor(decls);
+
+    int adjustment = 0;
+    QFont font;
+    extractor.extractFont(&font, &adjustment);
+
+    QFETCH(int, expectedPixelSize);
+    QFETCH(int, expectedPointSize);
+    QFETCH(qreal, expectedPointSizeF);
+
+    QCOMPARE(font.pixelSize(), expectedPixelSize);
+    QCOMPARE(font.pointSize(), expectedPointSize);
+    QCOMPARE(font.pointSizeF(), expectedPointSizeF);
+}
+
 void tst_QCssParser::extractBorder_data()
 {
     QTest::addColumn<QString>("css");
@@ -1655,10 +1725,10 @@ void tst_QCssParser::extractBorder()
     QCss::StyleSheet sheet;
     QVERIFY(parser.parse(&sheet));
 
-    QCOMPARE(sheet.styleRules.count() + sheet.nameIndex.count(), 1);
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size(), 1);
     QCss::StyleRule rule =  (!sheet.styleRules.isEmpty()) ?
             sheet.styleRules.at(0) : *sheet.nameIndex.begin();
-    const QVector<QCss::Declaration> decls = rule.declarations;
+    const QList<QCss::Declaration> decls = rule.declarations;
     QVERIFY(!decls.isEmpty());
     QCss::ValueExtractor extractor(decls);
 
@@ -1685,10 +1755,10 @@ void tst_QCssParser::noTextDecoration()
     QCss::StyleSheet sheet;
     QVERIFY(parser.parse(&sheet));
 
-    QCOMPARE(sheet.styleRules.count() + sheet.nameIndex.count(), 1);
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size(), 1);
     QCss::StyleRule rule =  (!sheet.styleRules.isEmpty()) ?
             sheet.styleRules.at(0) : *sheet.nameIndex.begin();
-    const QVector<QCss::Declaration> decls = rule.declarations;
+    const QList<QCss::Declaration> decls = rule.declarations;
     QVERIFY(!decls.isEmpty());
     QCss::ValueExtractor extractor(decls);
 
@@ -1710,10 +1780,10 @@ void tst_QCssParser::quotedAndUnquotedIdentifiers()
     QCss::StyleSheet sheet;
     QVERIFY(parser.parse(&sheet));
 
-    QCOMPARE(sheet.styleRules.count() + sheet.nameIndex.count(), 1);
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size(), 1);
     QCss::StyleRule rule = (!sheet.styleRules.isEmpty()) ?
            sheet.styleRules.at(0) : *sheet.nameIndex.begin();
-    const QVector<QCss::Declaration> decls = rule.declarations;
+    const QList<QCss::Declaration> decls = rule.declarations;
     QCOMPARE(decls.size(), 2);
 
     QCOMPARE(decls.at(0).d->values.first().type, QCss::Value::String);
@@ -1725,6 +1795,84 @@ void tst_QCssParser::quotedAndUnquotedIdentifiers()
     QCOMPARE(decls.at(1).d->values.first().toString(), QLatin1String("bold"));
 }
 
+void tst_QCssParser::whitespaceValues_data()
+{
+    QTest::addColumn<QString>("value");
+
+    QTest::newRow("normal") << "normal";
+    QTest::newRow("inherit") << "inherit";
+    QTest::newRow("nowrap") << "nowrap";
+    QTest::newRow("pre") << "pre";
+    QTest::newRow("pre-wrap") << "pre-wrap";
+    QTest::newRow("pre-line") << "pre-line";
+}
+
+void tst_QCssParser::whitespaceValues()
+{
+    QFETCH(QString, value);
+    QCss::Parser parser(QString("foo { white-space: %1 }").arg(value));
+    QCss::StyleSheet sheet;
+    QVERIFY(parser.parse(&sheet));
+
+    QCss::StyleRule rule = (!sheet.styleRules.isEmpty()) ?
+           sheet.styleRules.at(0) : *sheet.nameIndex.begin();
+    QCOMPARE(rule.declarations.size(), 1);
+
+    QCOMPARE(rule.declarations.at(0).d->property, QLatin1String("white-space"));
+    QCOMPARE(rule.declarations.at(0).d->values.first().toString(), value);
+}
+
+void tst_QCssParser::borderColor_data()
+{
+    QTest::addColumn<QString>("css");
+    QTest::addColumn<QColor>("expectedTopColor");
+    QTest::addColumn<QColor>("expectedRightColor");
+    QTest::addColumn<QColor>("expectedBottomColor");
+    QTest::addColumn<QColor>("expectedLeftColor");
+
+    QTest::newRow("four values") << "border-color: red green blue white" << QColor("red") << QColor("green") << QColor("blue") << QColor("white");
+    QTest::newRow("three values") << "border-color: red green blue" << QColor("red") << QColor("green") << QColor("blue") << QColor("green");
+    QTest::newRow("two values") << "border-color: red green" << QColor("red") << QColor("green") << QColor("red") << QColor("green");
+    QTest::newRow("one value") << "border-color: red" << QColor("red") << QColor("red") << QColor("red") << QColor("red");
+}
+
+void tst_QCssParser::borderColor()
+{
+    QFETCH(QString, css);
+    QFETCH(QColor, expectedTopColor);
+    QFETCH(QColor, expectedRightColor);
+    QFETCH(QColor, expectedBottomColor);
+    QFETCH(QColor, expectedLeftColor);
+
+    css.prepend("dummy {");
+    css.append(QLatin1Char('}'));
+
+    QCss::Parser parser(css);
+    QCss::StyleSheet sheet;
+    QVERIFY(parser.parse(&sheet));
+
+    QCOMPARE(sheet.styleRules.size() + sheet.nameIndex.size(), 1);
+    QCss::StyleRule rule =  (!sheet.styleRules.isEmpty()) ?
+            sheet.styleRules.at(0) : *sheet.nameIndex.begin();
+    const QList<QCss::Declaration> decls = rule.declarations;
+    QVERIFY(decls.size() == 1);
+    QVERIFY(decls[0].d->propertyId == QCss::BorderColor);
+
+    QBrush colors[4];
+
+    decls[0].brushValues(colors);
+    QCOMPARE(colors[QCss::TopEdge].color(), expectedTopColor);
+    QCOMPARE(colors[QCss::RightEdge].color(), expectedRightColor);
+    QCOMPARE(colors[QCss::BottomEdge].color(), expectedBottomColor);
+    QCOMPARE(colors[QCss::LeftEdge].color(), expectedLeftColor);
+
+    //QTBUG-126381 : a second evaluation should give the same results
+    QCOMPARE(colors[QCss::TopEdge].color(), expectedTopColor);
+    QCOMPARE(colors[QCss::RightEdge].color(), expectedRightColor);
+    QCOMPARE(colors[QCss::BottomEdge].color(), expectedBottomColor);
+    QCOMPARE(colors[QCss::LeftEdge].color(), expectedLeftColor);
+}
+
 QTEST_MAIN(tst_QCssParser)
 #include "tst_qcssparser.moc"
 
